package com.cultural.heritage.controller.file;

import cn.hutool.core.io.FileUtil;
import com.cultural.heritage.common.BaseResponse;
import com.cultural.heritage.common.ErrorCode;
import com.cultural.heritage.common.ResultUtils;
import com.cultural.heritage.constant.FileConstant;
import com.cultural.heritage.exception.BusinessException;
import com.cultural.heritage.model.dto.file.UploadFileRequest;
import com.cultural.heritage.model.entity.User;
import com.cultural.heritage.model.enums.FileUploadBizEnum;
import com.cultural.heritage.service.file.IHweiYunOBSService;
import com.cultural.heritage.service.user.UserService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.http.HttpServletRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.Arrays;

@RestController
@RequestMapping("/file")
@Slf4j
@Tag(name = "文件上传接口")
public class FileController {


    @Resource
    private UserService userService;


    @Resource
    private IHweiYunOBSService iHweiYunOBSService;


    /**
     * 未登录的情况下的文件上传
     * @param multipartFile      文件上传参数
     * @param uploadFileRequest  文件业务类型请求体
     * @return  图片可访问地址
     */
    @PostMapping("/upload/server/not_login")
    @Operation(summary = "(未使用)未登录情况下的文件上传", description = "参数：(文档上是false，但biz是必传的)，文件对象（multipartFile), 业务类型(biz)，权限：所有人，方法名：uploadServerFileNotLogin")
    private BaseResponse<String> uploadServerFileNotLogin(@RequestPart("file")MultipartFile multipartFile, UploadFileRequest uploadFileRequest) {
        //获取业务名称
        String biz = uploadFileRequest.getBiz();
        FileUploadBizEnum fileUploadBizEnum = FileUploadBizEnum.getEnumByValue(biz);
        if (fileUploadBizEnum == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "业务名称错误");
        }
        // 校验文件
        validFile(multipartFile, fileUploadBizEnum);
        //文件目录：根据业务、用户来划分
        String uuid = RandomStringUtils.randomAlphabetic(8);
        String filename = uuid + "-" + multipartFile.getOriginalFilename();
        String filepath = String.format("/%s/%s/%s", fileUploadBizEnum.getValue(), 0, filename);

        // 判断目录是否存在
        File file = new File(FileConstant.SERVER_UPLOAD_DIR, filepath);
        if (!file.exists()) {
            // 创建目录
            boolean mkdirs = file.mkdirs();
            if (!mkdirs) {
                throw new BusinessException(ErrorCode.SYSTEM_ERROR, "创建目录失败");
            }
        }
        //返回可访问地址
        String url = iHweiYunOBSService.fileUpload(multipartFile, "feiyi" + filepath);
        return ResultUtils.success(url);
    }


    @PostMapping("/upload/server")
    @Operation(summary = "已登录情况下的文件上传", description = "参数：(文档上是false，但biz是必传的)，文件对象（multipartFile), 业务类型(biz)，权限：所有人，方法名：uploadServerFile")
    public BaseResponse<String> uploadServerFile(@RequestPart("file")MultipartFile multipartFile, UploadFileRequest uploadFileRequest, HttpServletRequest request) {
        //获取业务名称
        String biz = uploadFileRequest.getBiz();
        FileUploadBizEnum fileUploadBizEnum = FileUploadBizEnum.getEnumByValue(biz);
        if (fileUploadBizEnum == null) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "业务名称错误");
        }
        // 校验文件
        validFile(multipartFile, fileUploadBizEnum);
        // 校验用户是否登录
        User loginUser = userService.getLoginUser(request);
        if (loginUser == null) {
            throw new BusinessException(ErrorCode.NOT_LOGIN_ERROR, "未登录");
        }
        // 文件目录：根据业务、用户来划分
        String uuid = RandomStringUtils.randomAlphabetic(8);
        String filename = uuid + "-" + multipartFile.getOriginalFilename();
        String filepath = String.format("/%s/%s/%s", fileUploadBizEnum.getValue(), loginUser.getId(), filename);

        // 判断目录是否存在
        File file = new File(FileConstant.SERVER_UPLOAD_DIR, filepath);
        if (!file.exists()) {
            // 创建目录
            boolean mkdirs = file.mkdirs();
            if (!mkdirs) {
                throw new BusinessException(ErrorCode.SYSTEM_ERROR, "创建目录失败");
            }
        }
        //返回可访问地址
        String url = iHweiYunOBSService.fileUpload(multipartFile, "feiyi" + filepath);
        return ResultUtils.success(url);
    }


    /**
     * 校验文件
     *
     * @param multipartFile      文件
     * @param fileUploadBizEnum  业务类型
     */
    private void validFile(MultipartFile multipartFile, FileUploadBizEnum fileUploadBizEnum) {
        //文件大小
        long fileSize = multipartFile.getSize();
        //文件后缀
        String fileSuffix = FileUtil.getSuffix(multipartFile.getOriginalFilename());
        final long LIMIT =  30 * 1024 * 1024L;
        if (fileSize > LIMIT) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "文件大小不能超过512K");
        }
        if (!Arrays.asList("jpeg", "jpg", "svg", "png", "webp", "JPG").contains(fileSuffix)) {
            throw new BusinessException(ErrorCode.PARAMS_ERROR, "文件类型错误");
        }
    }


}
